/* Name: main.c
 * Author: Michael Ciuffo
 * Published under Creative Commons Attribution-NonCommercial 3.0 Unported license
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>  

//Generated by hand
const uint8_t hello[53] PROGMEM = {0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0XFF,0XFF,0X18,0X18,0XFF,0XFF,0X00,0X00,0XFF,0XFF,0XDB,0XDB,0XC3,0X00,0X00,0XFF,0XFF,0X03,0X03,0X00,0X00,0XFF,0XFF,0X03,0X03,0X00,0X00,0X7E,0XFF,0XC3,0XC3,0XFF,0X7E,0X00,0X00,0X00};
const uint8_t goodbye[83] PROGMEM = {0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X7E,0X7E,0XFF,0XFF,0XC3,0XC3,0XCB,0XCB,0X4F,0X4F,0X4F,0X00,0X00,0X7E,0XFF,0XC3,0XC3,0XFF,0X7E,0X00,0X00,0X7E,0XFF,0XC3,0XC3,0XFF,0X7E,0X00,0X00,0XFF,0XFF,0XC3,0XC3,0XFF,0X7E,0X00,0X00,0XFF,0XFF,0XDB,0XDB,0XFF,0X6E,0X00,0X00,0XE0,0XF0,0X1F,0X1F,0XF0,0XE0,0X00,0X00,0XFF,0XFF,0XDB,0XDB,0XC3,0X00,0X00,0X00,0x00,0x00};

uint8_t arraylength;
uint8_t count;
uint32_t totalcount;
uint8_t backwards;
uint8_t currentrow;
uint8_t rowcount;
uint32_t totalcountminusone;
uint32_t totalcountminustwo;
uint8_t ishello;

int main(void)
{
	PORTB = 0XFB;
	PORTA = 0b10111111;

	DDRA = 0b10111111;
	DDRB = 0X03;
	    cli();
	
	totalcount = 0; //total number of interrupts since start of pattern
	totalcountminusone = 0;
	totalcountminustwo=0;
	backwards = 0; //is text displaying backwards?
	count = 0; //number of interrupts since last row was displayed
	currentrow = 0; //current row being displayed
	rowcount = 0; //number of interrupts between rows.
	arraylength = 53; //length of message array, set to "hello" array by default
	ishello = 1;//display hello by default
	
	OCR0A = 0x10; //how high you count
	TCCR0A = 0x02; //no output, mode 4 (clear timer on compare)
	TCCR0B = 0x01; //no prescaling.
	TIMSK0 = 0x02; //enable compare match interrupt.
	TCNT0 = 0;
	
	MCUCR = 1; //rising edge interrupt
	GIMSK = 0x40; //INT0 enabled
	sei();
	

    for(;;){}
    return 0;   /* never reached */
}

void printrow(uint8_t row)
{
	uint8_t temp;
	if (ishello)
	temp = ~pgm_read_byte(&(hello[row]));
	else
	temp = ~pgm_read_byte(&(goodbye[row]));

	PORTA = ((temp>>2)&0b00111111);
	PORTB = ((temp&0b00000011)|0b00000000);
	return;
}


//Fires either 83 or 53 times per half-cycle depending on whether "hello" or
//"goodbye" is being shown.  Each time it displays the subsequent row of the image...
ISR(TIM0_COMPA_vect)
{
	if (totalcount<100000)
		totalcount++;
	if (count < rowcount)//...until it runs out of image.
	{
		count++;
	}
	else 
	{
		count = 0;
		
		if ((currentrow < (arraylength-1))&&((PINB & 0B00000100)==4))
		{
			currentrow++;
			printrow(currentrow);
		}
				 

	}
	
}

//Fires once per cycle (when a pulse comes from the IR sensor)
ISR (INT0_vect)
{
    //select hello or goodbye depending on switch position
	if (PINA & 0b01000000)
	{
		ishello = 0;
		arraylength = 83;
	}
	else
	{
		ishello = 1;
		arraylength = 53;
	}
	
	uint32_t mastercount;
    //if you're on the return swing, don't do anything
	if ((PINB & 0B00000100)==4)
	{
		currentrow = 0;
		totalcount = 0;
		count = 0;
	}
    //otherwise, reset your clock and determine timing for next cycle
	else
	{
		currentrow = 0;
		printrow(currentrow);
		
		mastercount = totalcount;
		rowcount = mastercount/arraylength;

		
		
	}

	GIFR = (1<<INTF0);	//clear interrupt flag
}